#include <stdio.h>
#include <sys/time.h>
#include "rdma.h"

int main(int argc, char *argv[])
{
	if (argc != 2) {
		fprintf(stderr, "%s ip\n", argv[0]);
		return 1;
	}
	struct ha_cfg cfg = { .ip = argv[1],
			      .port = 8888,
			      .max_wr = 100,
			      .max_sge = 10,
			      .cap_send_wr = 10 };
	struct ha_connector *conn = NULL;
	struct timeval tv;
	int rc = ha_client(&cfg, &conn, 1000);

	if (rc) {
		perror("ha_client");
		return 1;
	}
	rc = ha_connect(conn, 1000);
	if (rc) {
		perror("ha_connect");
		return 1;
	}

	int a = 1;
	int b = 2;
	struct iovec iov[2];

	iov[0].iov_base = &a;
	iov[0].iov_len = sizeof(int);
	iov[1].iov_base = &b;
	iov[1].iov_len = sizeof(int);
	ha_regmrv(conn, iov, 2);

	ha_post_recv(conn, iov, 2);

	int count = 0;

	do {
		struct ibv_wc wc;

		bzero(&wc, sizeof(wc));
		int rc = ha_poll(conn, &wc, 1);

		if (rc < 0) {
			perror("ha_poll");
			break;
		}
		if (rc == 0)
			continue;

		struct ha_wrid *id = (struct ha_wrid *)wc.wr_id;
		id->iov_cb(id);
		if (wc.status != IBV_WC_SUCCESS) {
			printf("%s\n", ibv_wc_status_str(wc.status));
			break;
		}
		bzero(&tv, sizeof(tv));
		gettimeofday(&tv, NULL);
		printf("time: %ld\n", tv.tv_sec);
		printf("%d, %d\n", *((int *)iov[0].iov_base),
		       *((int *)iov[1].iov_base));
		count += 1;

		ha_deregmr(conn);
		ha_regmrv(conn, iov, 2);
		ha_post_recv(conn, iov, 2);
	} while (count < 2);

	ha_destroy_conn(conn);
	ha_deregmr(conn);
	return 0;
}
