/* SPDX-License-Identifier: Proprietary
 *
 * Yunhe Enmo (Beijing) Information Technology Co., Ltd.
 * Copyright (c) 2020 , All rights reserved.
 *
 */

#ifndef Z_RDMA_H_
#define Z_RDMA_H_

#include <stdbool.h>
#include <infiniband/verbs.h>
#include <rdma/rdma_cma.h>
#include <sys/queue.h>

#ifdef __cplusplus
extern "C" {
#endif

struct ha_cfg {
	const char *ip;
	unsigned short port;
	size_t max_wr;
	size_t max_sge;
	size_t cap_send_wr;
};

enum ha_opcode { SEND = 1, RECV };

struct ha_wrid;

typedef void (*iov_cb)(struct ha_wrid *id);

struct ha_wrid {
	enum ha_opcode type;
	// identifier for connected(ha_connector::connected == true) qp
	struct ha_connector *ctx;
	struct ibv_sge *sge;
	int n;
	iov_cb iov_cb;

	TAILQ_ENTRY(ha_wrid) next;

	// reserved for user, never used by this library
	void *user_ctx;
};

TAILQ_HEAD(ha_wrid_que, ha_wrid);

struct ha_connector;

struct ha_acceptor {
	struct ha_cfg cfg;
	// managed by ha_regmr/ha_deregmr
	struct mr_queue *mr_q;
	struct ibv_cq *cq;
	// set to 1 at presnet, shared among connections created by ha_accept
	// number of srq is control by cfg.max_srq
	struct ibv_srq *srq;
	struct rdma_event_channel *chan;
	// never associate to a qp
	struct rdma_cm_id *cm_id;

	// connector for post_recv
	struct ha_connector *conn;
};

struct ha_connector {
	bool connected;
	uint32_t qp_num;
	struct ha_cfg cfg;
	struct mr_queue *mr_q;
	struct ibv_cq *cq;
	struct ibv_srq *srq;
	struct rdma_cm_id *cm_id;
	struct rdma_event_channel *chan;
	struct ha_acceptor *acceptor;

	struct ha_wrid_que head;
};

int ha_regmrv(struct ha_connector *ctx, struct iovec *iov, int n);

int ha_deregmr(struct ha_connector *ctx);

struct ha_acceptor *ha_server(struct ha_cfg *cfg);

int ha_listen(struct ha_acceptor *ctx);

int ha_accept(struct ha_acceptor *ctx, struct ha_connector **conn);

int ha_client(struct ha_cfg *cfg, struct ha_connector **conn, int timeout);

int ha_connect(struct ha_connector *ctx, int timeout);

bool ha_connected(struct ha_connector *ctx);

int ha_conn_status(struct ha_connector *ctx, struct ibv_qp_attr *attr,
		   struct ibv_qp_init_attr *init_attr);

void ha_destroy_conn(struct ha_connector *ctx);

void ha_destroy_acceptor(struct ha_acceptor *ctx);

int ha_poll(struct ha_connector *ctx, struct ibv_wc *wc, int nwc);

int ha_post_send(struct ha_connector *ctx, const struct iovec *iov, int n);

int ha_post_recv(struct ha_connector *ctx, const struct iovec *iov, int n);

#ifdef __cplusplus
}
#endif

#endif
