#include <stdio.h>
#include <sys/time.h>
#include "rdma.h"

int main(int argc, char *argv[])
{
	if (argc != 2) {
		fprintf(stderr, "%s ip\n", argv[0]);
		return 1;
	}
	struct ha_cfg cfg = { .ip = argv[1],
			      .port = 8888,
			      .max_wr = 100,
			      .max_sge = 10,
			      .cap_send_wr = 10 };
	struct ha_acceptor *acceptor = ha_server(&cfg);
	struct timeval tv;

	ha_listen(acceptor);

	struct ha_connector *conn = NULL;

	do {
		ha_accept(acceptor, &conn);
	} while (!conn);

	int a = 12;
	int b = 34;
	int c = 21;
	int d = 43;
	struct iovec iov[2];
	struct iovec iov2[2];

	iov[0].iov_base = &a;
	iov[0].iov_len = sizeof(int);
	iov[1].iov_base = &b;
	iov[1].iov_len = sizeof(int);
	ha_regmrv(acceptor->conn, iov, 2);

	iov2[0].iov_base = &c;
	iov2[0].iov_len = sizeof(int);
	iov2[1].iov_base = &d;
	iov2[1].iov_len = sizeof(int);
	ha_regmrv(acceptor->conn, iov2, 2);

	ha_post_send(conn, iov, 2);
	ha_post_send(conn, iov2, 2);

	int count = 0;

	do {
		struct ibv_wc wc;

		bzero(&wc, sizeof(wc));
		int rc = ha_poll(conn, &wc, 1);

		if (rc < 0) {
			perror("ha_poll");
			break;
		}
		if (rc == 0)
			continue;

		struct ha_wrid *id = (struct ha_wrid *)wc.wr_id;
		id->iov_cb(id);
		if (wc.status != IBV_WC_SUCCESS) {
			printf("%s\n", ibv_wc_status_str(wc.status));
			break;
		}
		bzero(&tv, sizeof(tv));
		gettimeofday(&tv, NULL);
		printf("time: %ld\n", tv.tv_sec);
		if (count == 0)
			printf("%d, %d\n", *((int *)iov[0].iov_base),
			       *((int *)iov[1].iov_base));
		else
			printf("%d, %d\n", *((int *)iov2[0].iov_base),
			       *((int *)iov2[1].iov_base));
		count += 1;
	} while (count < 2);

	ha_destroy_conn(conn);
	ha_deregmr(acceptor->conn);
	ha_destroy_acceptor(acceptor);
	return 0;
}
